--- wine/windows/msgbox.c	2004-11-04 22:53:48.387478944 +0000
+++ new/windows/msgbox.c	2004-11-04 22:52:31.697137648 +0000
@@ -34,11 +34,22 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(dialog);
 WINE_DECLARE_DEBUG_CHANNEL(msgbox);
+BOOL DIALOG_DisableOwner(HWND hOwner);
+void DIALOG_EnableOwner(HWND hOwner);
 
 #define MSGBOX_IDICON 1088
 #define MSGBOX_IDTEXT 100
 #define IDS_ERROR     2
 
+BOOL CALLBACK MSGBOX_EnumProc(HWND hwnd, LPARAM lParam)
+{
+    if(lParam==0)
+        DIALOG_DisableOwner(hwnd);
+    if(lParam==1)
+        DIALOG_EnableOwner(hwnd);
+    return TRUE;
+}
+
 static HFONT MSGBOX_OnInit(HWND hwnd, LPMSGBOXPARAMSW lpmb)
 {
     HFONT hFont = 0, hPrevFont = 0;
@@ -248,20 +259,18 @@
 	}
     }
 
-    /* handle modal MessageBoxes */
-    if (lpmb->dwStyle & MB_SYSTEMMODAL)
-        SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
-    
-    if (lpmb->dwStyle & MB_TASKMODAL)
+    /*habdle task modal message boxes*/
+    if ((lpmb->dwStyle & MB_TASKMODAL) && (lpmb->hwndOwner==NULL))
     {
-	FIXME("task modal msgbox ! Not modal yet.\n");
-	/* Probably do EnumTaskWindows etc. here and work
-	 * your way up to the top - I'm lazy (HWND_TOP) */
-	SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
-	/* MB_TASKMODAL seems to imply a ShowWindow */
-	ShowWindow(hwnd, SW_SHOW);
+        EnumThreadWindows(GetCurrentThreadId(),  MSGBOX_EnumProc, 0);
+        DIALOG_EnableOwner(hwnd);
+        SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
     }
 
+    /* handle system modal MessageBoxes */
+    if (lpmb->dwStyle & MB_SYSTEMMODAL)
+        SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
+
     return hFont;
 }
 
@@ -443,6 +452,7 @@
 {
     LPVOID tmplate;
     HRSRC hRes;
+    int ret;
     static const WCHAR msg_box_res_nameW[] = { 'M','S','G','B','O','X',0 };
 
     if (!(hRes = FindResourceExW(user32_module, (LPWSTR)RT_DIALOG,
@@ -451,6 +461,13 @@
     if (!(tmplate = (LPVOID)LoadResource(user32_module, hRes)))
         return 0;
 
-    return DialogBoxIndirectParamW(msgbox->hInstance, tmplate, msgbox->hwndOwner,
-                                   MSGBOX_DlgProc, (LPARAM)msgbox);
+    ret=DialogBoxIndirectParamW(msgbox->hInstance, tmplate, msgbox->hwndOwner, MSGBOX_DlgProc, (LPARAM)msgbox);
+
+    /*re-enable top level windows if the messagebox is taskmodal*/
+    if ((msgbox->dwStyle & MB_TASKMODAL) && (msgbox->hwndOwner==NULL))
+    {
+        EnumThreadWindows(GetCurrentThreadId(),  MSGBOX_EnumProc, 1);
+    }
+
+    return ret;
 }
